@@ -14,6 +14,7 @@ from libreflow.baseflow.file import (
     AddFilesFromExisting,
     FileRevisionNameChoiceValue,
 )
+from libreflow.baseflow.dependency import GetDependenciesAction
 
 from kabaret.flow_contextual_dict import get_contextual_dict
 
@@ -226,6 +227,7 @@ class PublishOKItem(flow.Object):
 class TrackedFile(TrackedFile, PublishOKItem):
 
     publish_ok = flow.Child(PublishOKAction).ui(group='Advanced')
+    get_dependencies = flow.Child(GetDependenciesAction).ui(group='Advanced')
 
     def get_name(self):
         # Two remarks:
@@ -242,6 +244,58 @@ class TrackedFile(TrackedFile, PublishOKItem):
         else:
             super(TrackedFile, self).compute_child_value(child_value)
 
+    def get_dependency_template(self):
+        kitsu_bindings = self.root().project().kitsu_bindings()
+        settings = get_contextual_dict(self, 'settings')
+        file_category = settings['file_category']
+        casting = dict()
+
+        if file_category == 'PROD':
+            shot = settings['shot']
+            sequence = settings['sequence']
+            film = settings['film']
+            casting = kitsu_bindings.get_shot_casting(shot, sequence)
+            entity_oid = self.root().project().oid()
+            entity_oid += f"/films/{film}/sequences/{sequence}/shots/{shot}"
+        elif file_category == 'LIB':
+            asset_name = settings['asset_name']
+            asset_family = settings['asset_family']
+            asset_type = settings['asset_type']
+            entity_oid = self.root().project().oid()
+            entity_oid += f"/asset_lib/asset_types/{asset_type}/asset_families/{asset_family}/assets/{asset_name}"
+
+        dependency_template = settings.get('dependency_template', self.name())
+
+        return dependency_template, entity_oid, casting
+
+    def get_real_dependencies(self, revision_name=None):
+        if revision_name is None:
+            revision = self.get_head_revision()
+        else:
+            revision = self.get_revision(revision_name)
+
+        # Return if revision does not exist
+        if not revision:
+            return []
+        
+        dependencies = revision.dependencies.get()
+        
+        # Return if no dependency has been reported for this revision
+        if not dependencies:
+            return []
+
+        deps_file_paths = set()
+        
+        for dep_path, file_paths in dependencies.items():
+            deps_file_paths.add(dep_path)
+            dep_dir = os.path.dirname(dep_path)
+            
+            for path in file_paths:
+                resolved_path = os.path.normpath(os.path.join(dep_dir, path))
+                deps_file_paths.add(resolved_path)
+
+        return list(deps_file_paths)
+
 
 class TrackedFolder(TrackedFolder, PublishOKItem):
     
