@@ -2,6 +2,7 @@ import os
 import gazu
 import shutil
 import glob
+import time
 
 import kabaret.app.resources as resources
 from kabaret import flow
@@ -15,6 +16,7 @@ from libreflow.utils.b3d import wrap_python_expr
 class Department(baseflow.departments.Department):
     _short_name = flow.Param(None)
     _file_prefix = flow.Computed(cached=True)
+    _shot = flow.Parent(2)
     
     def compute_child_value(self, child_value):
         if child_value is self.path:
@@ -35,7 +37,7 @@ class Department(baseflow.departments.Department):
             return dict(
                 department=self.name(),
                 dept=self._short_name.get() if self._short_name.get() else self.name(),
-                context=self._parent.__class__.__name__.lower(),
+                context=self._shot.__class__.__name__.lower(),
             )
 
 
@@ -48,51 +50,41 @@ class LayoutDependency(flow.Object):
 
     _parent = flow.Parent()
     
-    asset_name = flow.Computed()
-    asset_type = flow.Computed()
-    asset_family = flow.Computed()
-    asset_path = flow.Computed()
-    asset_file_oid = flow.Computed()
-    asset_revision_oid = flow.Computed()
-    available = flow.Computed()
+    asset_type = flow.Computed(store_value=False)
+    asset_family = flow.Computed(store_value=False)
+    asset_number = flow.Computed(store_value=False)
+    asset_path = flow.Computed(store_value=False)
+    asset_file_oid = flow.Computed(store_value=False)
+    asset_revision_oid = flow.Computed(store_value=False)
+    available = flow.Computed(store_value=False)
 
     def compute_child_value(self, child_value):
-        if child_value is self.asset_name:
-            child_value.set(self.name())
-        elif child_value is self.asset_type:
-            if self.name() in ["audio", "board"]:
-                child_value.set(self.name())
-                return
+        asset_data = self._parent.asset_data(self.name())
 
-            child_value.set(
-                self._parent.asset_type_short_name(
-                    self._parent.asset_data(self.name())["asset_type_name"]
-                )
-            )
+        if child_value is self.asset_type:
+            child_value.set(asset_data['asset_type'])
         elif child_value is self.asset_family:
-            if self.name() in ["audio", "board"]:
-                child_value.set(self.name())
-                return
-
-            kitsu_api = self.root().project().kitsu_api()
-            child_value.set(self._parent.asset_family_short_name(
-                kitsu_api.get_asset_data(self.name())["data"]["family"]
-            ))
+            child_value.set(asset_data['asset_family'])
+        elif child_value is self.asset_number and 'asset_number' in asset_data:
+            child_value.set(asset_data['asset_number'])
         elif child_value is self.asset_file_oid:
+            asset_type = self.asset_type.get()
             asset = None
 
             if self.name() in ["audio", "board"]:
-                asset_type = self.name()
                 asset = self._parent._shot
             else:
-                asset_type = self.asset_type.get()
                 asset_family = self.asset_family.get()
-                asset_name = self.asset_name.get()
+                asset_name = self.name()
                 asset_oid = self.root().project().oid() + f"/asset_lib/asset_types/{asset_type}/asset_families/{asset_family}/assets/{asset_name}"
 
                 if self.root().session().cmds.Flow.exists(asset_oid):
                     asset = self.root().get_object(asset_oid)
-            
+                else:
+                    self.root().session().log_warning(
+                        f"Asset {asset_oid} not found"
+                    )
+
             if not asset:
                 child_value.set(None)
                 return
@@ -149,29 +141,44 @@ class LayoutDependencies(flow.DynamicMap):
     _sequence = flow.Parent(6)
     _updated = flow.BoolParam(False)
 
+    def __init__(self, parent, name):
+        super(LayoutDependencies, self).__init__(parent, name)
+        self._assets_data_time = time.time()
+        self._assets_data = None
+
     def mapped_names(self, page_num=0, page_size=None):
-        kitsu_api = self.root().project().kitsu_api()
-        shot_name = self._shot.name()
-        sequence_name = self._sequence.name()
+        if not self._assets_data or time.time() - self._assets_data_time > 30.0:
+            self._assets_data = self._get_assets_data()
+            self._assets_data_time = time.time()
 
-        # Get shot data
-        kitsu_api = self.root().project().kitsu_api()
-        shot_data = kitsu_api.get_shot_data(shot_name, sequence_name)
-        self._assets_data = {}
+        return list(self._assets_data.keys())
 
-        if not shot_data:
-            self.message.set((
-                "<h2>Configure layout shot</h2>"
-                "<font color=#D5000D>"
-                f'No shot named <b>{shot_name}</b> in sequence <b>{sequence_name}</b>.'
-                "</font>"
-            ))
-            return []
+    def _get_assets_data(self):
+        kitsu_api = self.root().project().kitsu_api()
+        kitsu_casting = kitsu_api.get_shot_casting(self._shot.name(), self._sequence.name())
+        casting = dict()
         
-        shot_casting = kitsu_api.get_shot_casting(shot_data)
-        self._assets_data = {asset["asset_name"]: asset for asset in shot_casting}
-        return list(self._assets_data.keys()) + ["audio", "board"]
-    
+        # Kitsu assets
+        for asset in kitsu_casting:
+            asset_name = asset['asset_name']
+            asset_type = asset['asset_type_name']
+            asset_family = kitsu_api.get_asset_data(asset_name)['data']['family']
+
+            casting[asset_name] = dict(
+                asset_type=self.asset_type_short_name(asset_type),
+                asset_family=self.asset_family_short_name(asset_family),
+                asset_number=asset['nb_occurences']
+            )
+        
+        # Audio file and storyboard
+        for asset_name in ['audio', 'board']:
+            casting[asset_name] = dict(
+                asset_type=asset_name,
+                asset_family='misc',
+            )
+        
+        return casting
+
     def asset_data(self, asset_name):
         return self._assets_data[asset_name]
 
@@ -213,7 +220,7 @@ class LayoutDependencies(flow.DynamicMap):
         return name
     
     def _fill_row_cells(self, row, item):
-        row["Name"] = item.asset_name.get()
+        row["Name"] = item.name()
         row["Type"] = item.asset_type.get()
         row["Family"] = item.asset_family.get()
         
@@ -230,13 +237,13 @@ class LayoutDependencies(flow.DynamicMap):
         style["icon"] = icon_by_status[item.available.get()]
 
 
-class BuildLayoutScene(RunAction):
+class BuildBlenderScene(RunAction):
 
     _department = flow.Parent()
     _shot = flow.Parent(3)
     _sequence = flow.Parent(5)
 
-    dependencies = flow.Child(LayoutDependencies)
+    dependencies = flow.Child(LayoutDependencies).ui(expanded=True)
 
     def runner_name_and_tags(self):
         return "Blender", []
@@ -244,17 +251,80 @@ class BuildLayoutScene(RunAction):
     def allow_context(self, context):
         return context and context.endswith(".inline")
 
-    def extra_argv(self):
-        # Declare Blender operator call generators
-        blender_operators = {
-            "setup": lambda **kwargs: 'bpy.ops.pipeline.scene_builder_setup(frame_start={frame_start}, frame_end={frame_end}, resolution_x={resolution_x}, resolution_y={resolution_y}, fps={fps})\n'.format(**kwargs),
-            "add_asset": lambda filepath, asset_name: f'bpy.ops.pipeline.scene_builder_import_asset(filepath="{filepath}", asset_name="{asset_name}")\n',
-            "add_set": lambda set_dir, set_dicts: f'bpy.ops.pipeline.scene_builder_import_set(directory="{set_dir}", files={set_dicts})\n',
-            "add_audio": lambda filepath: f'bpy.ops.pipeline.scene_builder_import_audio(filepath="{filepath}")\n',
-            "add_board": lambda filepath: f'bpy.ops.pipeline.scene_builder_import_storyboard(filepath="{filepath}")\n',
-            "save": lambda filepath: f'bpy.ops.wm.save_as_mainfile(filepath="{filepath}", compress=True)\n',
+    def extra_env(self):
+        return {
+            "ROOT_PATH": self.root().project().get_root()
         }
+    
+    def _ensure_file(self, name, format, to_edit=False, src_path=None, publish_comment=""):
+        files = self._department.files
+        file_name = "%s_%s" % (name, format)
+
+        if not files.has_mapped_name(file_name):
+            files.create_file.file_name.set(name)
+            files.create_file.file_format.set(format)
+            files.create_file.run(None)
         
+        file = files[file_name]
+
+        if not to_edit and not src_path:
+            return None
+        
+        if to_edit:
+            revision = file.create_working_copy(source_path=src_path)
+        else:
+            revision = file.publish(source_path=src_path, comment=publish_comment)
+        
+        file.set_current_user_on_revision(revision.name())
+
+        return revision.get_path()
+
+    def _blender_cmd(self, operator, **kwargs):
+        '''
+        Returns Blender scene builder operator command as a string.
+
+        Operator must be one of the following: `setup`, `setup_anim`,
+                                               `add_asset`, `add_set`, `add_audio`, `add_board`,
+                                               `update_audio", "update_storyboard`,
+                                               `remap_relative`, `save`.
+        '''
+
+        blender_operators = {
+            "setup": {'operator_command': "bpy.ops.pipeline.scene_builder_setup",
+                      'args': "frame_start={frame_start}, frame_end={frame_end}, resolution_x={resolution_x}, resolution_y={resolution_y}, fps={fps}"},
+            "setup_anim": {'operator_command': "bpy.ops.pipeline.scene_builder_setup_animation",
+                           'args': 'alembic_filepath="{alembic_filepath}", assets={assets}'},
+
+            "add_asset": {'operator_command': 'bpy.ops.pipeline.scene_builder_import_asset',
+                          'args': 'filepath="{filepath}", asset_name="{asset_name}", target_collection="{asset_type}"'},
+            "add_set": {'operator_command': 'bpy.ops.pipeline.scene_builder_import_set',
+                        'args': 'directory="{set_dir}", files={set_dicts}'},
+            "add_audio": {'operator_command': 'bpy.ops.pipeline.scene_builder_import_audio',
+                          'args': 'filepath="{filepath}"'},
+            "add_board": {'operator_command': 'bpy.ops.pipeline.scene_builder_import_storyboard',
+                          'args': 'filepath="{filepath}"'},
+
+            "update_audio": {'operator_command': 'bpy.ops.pipeline.scene_builder_update_audio',
+                             'args' :'filepath="{filepath}"'},
+            "update_board": {'operator_command': 'bpy.ops.pipeline.scene_builder_update_storyboard',
+                             'args' :'filepath="{filepath}"'},
+
+            "remap_relative": {'operator_command': 'bpy.ops.file.make_paths_relative',
+                               'args': ''},
+            "save": {'operator_command': 'bpy.ops.wm.save_mainfile',
+                     'args': 'filepath="{filepath}", compress=True'},
+        }
+
+        op = blender_operators[operator]
+        operator_command = op['operator_command']
+        args = op['args'].format(**kwargs)
+        command = f"if {operator_command}.poll(): {operator_command}({args})\n"
+        return command
+
+
+class BuildLayoutScene(BuildBlenderScene):
+
+    def extra_argv(self):
         # Get scene builder arguments
         frame_start = 101
         frame_end = 101 + self._shot_data.get("nb_frames", 0) - 1
@@ -271,28 +341,26 @@ class BuildLayoutScene(RunAction):
 
         # Build Blender Python expression
         python_expr = "import bpy\n"
-        python_expr += blender_operators["setup"](frame_start=frame_start, frame_end=frame_end, resolution_x=resolution_x, resolution_y=resolution_y, fps=fps)
+        python_expr += self._blender_cmd("setup", frame_start=frame_start, frame_end=frame_end, resolution_x=resolution_x, resolution_y=resolution_y, fps=fps)
+        python_expr += self._blender_cmd("save", filepath=layout_path)
         
-        for name, path in assets:
-            python_expr += blender_operators["add_asset"](path, name)
+        for name, path, asset_type, asset_number in assets:
+            for i in range(asset_number):
+                python_expr += self._blender_cmd("add_asset", filepath=path, asset_name=name, asset_type=asset_type)
         for set_dir, set_dicts in sets:
-            python_expr += blender_operators["add_set"](set_dir, set_dicts)
+            python_expr += self._blender_cmd("add_set", set_dir=set_dir, set_dicts=set_dicts)
 
-        if audio_path: python_expr += blender_operators["add_audio"](audio_path)
-        if board_path: python_expr += blender_operators["add_board"](board_path)
+        if audio_path: python_expr += self._blender_cmd("add_audio", filepath=audio_path)
+        if board_path: python_expr += self._blender_cmd("add_board", filepath=board_path)
 
-        python_expr += blender_operators["save"](layout_path)
+        python_expr += self._blender_cmd("remap_relative")
+        python_expr += self._blender_cmd("save", filepath=layout_path)
 
         return [
             "-b", template_path,
             "--addons", "io_import_images_as_planes,camera_plane,lfs_scene_builder,add_camera_rigs",
             "--python-expr", wrap_python_expr(python_expr)
         ]
-    
-    def extra_env(self):
-        return {
-            "ROOT_PATH": self.root().project().get_root()
-        }
 
     def get_buttons(self):
         msg = "<h2>Configure layout shot</h2>"
@@ -311,33 +379,6 @@ class BuildLayoutScene(RunAction):
 
         return ["Build and edit", "Build and publish", "Cancel"]
     
-    def _create_file(self, name, format, department):
-        files = department.files
-        files.create_file.file_name.set(name)
-        files.create_file.file_format.set(format)
-        files.create_file.run(None)
-        
-        return files["%s_%s" % (name, format)]
-
-    def _create_working_copy(self, file, original_path=None):
-        file.create_working_copy_action.from_revision.set("")
-        file.create_working_copy_action.run(None)
-        working_copy = file.get_working_copy()
-
-        if original_path:
-            working_copy_path = working_copy.get_path()
-            os.remove(working_copy_path)
-            shutil.copy2(original_path, working_copy_path)
-
-        return working_copy
-    
-    def _publish_file(self, file, comment):
-        file.publish_action.comment.set(comment)
-        file.publish_action.upload_after_publish.set(True)
-        file.publish_action.run("Unlock")
-        
-        return file.get_head_revision()
-    
     def run(self, button):
         if button == "Cancel":
             return
@@ -364,10 +405,11 @@ class BuildLayoutScene(RunAction):
                 continue
 
             asset_type = dep.asset_type.get()
+            asset_number = dep.asset_number.get()
             path = dep.asset_path.get().replace("\\", "/")
             
             if asset_type in ["chars", "props"]:
-                self._shot_data["assets_data"].append((dep.asset_name.get(), path))
+                self._shot_data["assets_data"].append((dep.name(), path, asset_type, asset_number))
             elif asset_type == "sets":
                 set_names = list(map(os.path.basename, glob.glob("%s/*.png" % path)))
 
@@ -378,17 +420,20 @@ class BuildLayoutScene(RunAction):
                 self._shot_data["%s_path" % asset_type] = path
 
         # Configure layout file
-        if not self._shot.departments.layout.files.has_mapped_name("layout_blend"):
-            layout_file = self._create_file("layout", "blend", self._department)
-        else:
-            layout_file = self._shot.departments.layout.files["layout_blend"]
-    
-        layout_revision = self._create_working_copy(layout_file)
-        if button == "Build and publish":
-            layout_revision = self._publish_file(layout_file, "Created with scene builder")
+        layout_path = self._ensure_file(
+            name='layout',
+            format='blend',
+            to_edit=(button == 'Build and edit'),
+            src_path=resources.get("file_templates", "template.blend"),
+            publish_comment="Created with scene builder"
+        )
+        
+        self._department.files.touch()
 
-        self._shot_data["layout_path"] = layout_revision.get_path().replace("\\", "/")
+        # Store layout output path
+        self._shot_data["layout_path"] = layout_path.replace("\\", "/")
 
+        # Build
         super(BuildLayoutScene, self).run(button)
 
 
@@ -399,15 +444,229 @@ class LayoutDepartment(Department):
         label="Build layout scene"
     )
 
+
+class BuildBlockingScene(BuildBlenderScene):
+
+    _department = flow.Parent()
+    _shot = flow.Parent(3)
+    _sequence = flow.Parent(5)
+    
+    def extra_argv(self):
+        # Get scene builder arguments
+        anim_path = self._shot_data.get("anim_path", None)
+        alembic_path = self._shot_data.get("alembic_path", None)
+        assets_data = self._shot_data.get("assets_data", [])
+        assets = []
+        for name, path, asset_type, asset_number in assets_data:
+            for i in range(asset_number):
+                assets.append({"name": name,
+                                "filepath": path,
+                                "target_collection": asset_type})
+        audio_path = self._shot_data.get("audio_path", None)
+        board_path = self._shot_data.get("board_path", None)
+
+        # Build Blender Python expression
+        python_expr = "import bpy\n"
+        python_expr += self._blender_cmd("setup_anim",
+                                         alembic_filepath=alembic_path, assets=assets)
+
+        # Update reference files
+
+        if audio_path:
+            python_expr += self._blender_cmd("update_audio", filepath=audio_path)
+        if board_path:
+            python_expr += self._blender_cmd("update_board", filepath=board_path)
+
+        python_expr += self._blender_cmd("remap_relative")
+        python_expr += self._blender_cmd("save", filepath=anim_path)
+
+        return [
+            "-b", anim_path,
+            "--addons", "io_import_images_as_planes,camera_plane,lfs_scene_builder,add_camera_rigs",
+            "--python-expr", wrap_python_expr(python_expr)
+        ]
+
+    def get_buttons(self):
+        latest_revision = None
+        files = self._shot.departments.layout.files
+
+        if "layout_blend" in files.mapped_names():
+            latest_revision = files["layout_blend"].get_head_revision(sync_status='Available')
+
+        if (latest_revision is None):
+            msg = "<h2><font color=#D5000D>Layout file not available</font></h2>"
+            buttons =  ["Cancel"]
+        else:
+            msg = "<h2>Configure animation shot</h2>"
+            buttons = ["Build and edit", "Build and publish", "Cancel"]
+
+        self.message.set(msg)
+
+        return buttons
+
+    def run(self, button):
+        if button == 'Cancel':
+            return
+
+        if button == "Refresh":
+            self.dependencies.touch()
+            return self.get_result(refresh=True, close=False)
+
+        shot_name = self._shot.name()
+        sequence_name = self._sequence.name()
+
+        # Store dependencies file paths for Blender script building
+        self._shot_data = {}
+        self._shot_data["assets_data"] = []
+
+        for dep in self.dependencies.mapped_items():
+            if dep.available.get() != "Available":
+                continue
+
+            asset_type = dep.asset_type.get()
+            path = dep.asset_path.get().replace("\\", "/")
+            asset_number = dep.asset_number.get()
+
+            if asset_type in ["chars", "props"]:
+                self._shot_data["assets_data"].append((dep.name(), path, asset_type, asset_number))
+            else:
+                self._shot_data["%s_path" % asset_type] = path
+
+        # Configure anim file
+        latest_revision = self._shot.departments.layout.files["layout_blend"].get_head_revision()
+        layout_path = latest_revision.get_path()
+
+        # Create empty file
+        anim_path = self._ensure_file(
+            name='blocking',
+            format='blend',
+            to_edit=(button == 'Build and edit'),
+            src_path=layout_path,
+            publish_comment="Created with anim scene builder"
+        )
+
+        # Configure alembic file
+        alembic_path = self._ensure_file(
+            name='ref_layout',
+            format='abc',
+            src_path=resources.get("file_templates", "template.abc"),
+            publish_comment="Created with anim scene builder"
+        )
+
+        self._department.files.touch()
+
+        # Store anim and alembic output path
+        self._shot_data["anim_path"] = anim_path.replace("\\", "/")
+        self._shot_data["alembic_path"] = alembic_path.replace("\\", "/")
+
+        # Build
+        super(BuildBlockingScene, self).run(button)
+
+
+class BlockingRevisionName(baseflow.file.FileRevisionNameChoiceValue):
+    
+    def get_file(self):
+        blocking_file_name = self.action.blocking_scene.get()
+        if blocking_file_name is None:
+            return None
+        
+        mapped_name = blocking_file_name.replace('.', '_')
+        return self.action._department.files[mapped_name]
+
+
+class CreateAnimationScene(flow.Action):
+    
+    _department = flow.Parent()
+    blocking_scene = flow.Computed()
+    revision_name = flow.Param(None, BlockingRevisionName)
+    
+    def get_buttons(self):
+        self.revision_name.revert_to_default()
+        return ['Create', 'Cancel']
+    
+    def needs_dialog(self):
+        return (
+            self.blocking_scene.get() is not None
+            and self.revision_name.choices()
+        )
+    
+    def _ensure_file(self, name, format):
+        files = self._department.files
+        mapped_name = '%s_%s' % (name, format)
+        
+        if not files.has_mapped_name(mapped_name):
+            files = self._department.files
+            files.create_file.file_name.set(name)
+            files.create_file.file_format.set(format)
+            files.create_file.run(None)
+        
+        return files[mapped_name]
+    
+    def compute_child_value(self, child_value):
+        if child_value is self.blocking_scene:
+            files = self._department.files
+            name = None
+            
+            if files.has_mapped_name('blocking_blend') and \
+                 files['blocking_blend'].get_head_revision() is not None:
+                name = 'blocking.blend'
+            elif files.has_mapped_name('breakdown_blend') and \
+                 files['breakdown_blend'].get_head_revision() is not None:
+                name = 'breakdown.blend'
+            
+            self.blocking_scene.set(name)
+    
+    def run(self, button):
+        if button == 'Cancel':
+            return
+        
+        # Ensure animation scene is created
+        files = self._department.files
+        anim_file = self._ensure_file('animation', 'blend')
+        
+        # Check for a blocking scene
+        blocking_file_name = self.blocking_scene.get()
+        if blocking_file_name is None:
+            files.touch()
+            return
+        
+        mapped_name = blocking_file_name.replace('.', '_')
+        blocking_file = files[mapped_name]
+
+        # Create a working copy from it
+        revisions = blocking_file.get_revisions()
+        revision_name = self.revision_name.get()
+        
+        if not revisions.has_mapped_name(revision_name):
+            files.touch()
+            return
+        
+        working_copy = anim_file.create_working_copy(
+            source_path=revisions[revision_name].get_path()
+        )
+        anim_file.set_current_user_on_revision(working_copy.name())
+        anim_file.touch()
+        
+        files.touch()
+
+
 class AnimationDepartment(Department):
     _short_name = flow.Param("ani")
 
+    build_blocking_scene = flow.Child(BuildBlockingScene)
+    create_anim_scene = flow.Child(CreateAnimationScene)
+
+
+class CompDepartment(Department):
+    _short_name = flow.Param("comp")
+
 class MiscDepartment(Department):
     _short_name = flow.Param("misc")
 
 class ShotDepartments(flow.Object):
     layout = flow.Child(LayoutDepartment).ui(expanded=False)
     animation = flow.Child(AnimationDepartment).ui(expanded=False)
+    compositing = flow.Child(CompDepartment).ui(expanded=False)
     misc = flow.Child(MiscDepartment).ui(expanded=False)
 
 
@@ -435,7 +694,11 @@ class Shots(baseflow.film.Shots):
 class Sequence(baseflow.film.Sequence):
 
     _film = flow.Parent(2)
-    shots = flow.Child(Shots).ui(default_height=420, expanded=True)
+    shots = flow.Child(Shots).ui(
+        default_height=420,
+        expanded=True,
+        show_filter=True,
+    )
 
     def compute_child_value(self, child_value):
         if child_value is self.kitsu_url:
@@ -467,12 +730,20 @@ class Film(flow.Object):
 
     ICON = ("icons.flow", "film")
 
-    sequences = flow.Child(Sequences).ui(default_height=420, expanded=True)
+    sequences = flow.Child(Sequences).ui(
+        default_height=420,
+        expanded=True,
+        show_filter=True,
+    )
     
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
             return dict(film=self.name())
 
+    def _fill_ui(self, ui):
+        if self.root().project().show_login_page():
+            ui["custom_page"] = "libreflow.baseflow.LoginPageWidget"
+
 
 class Films(flow.Map):
 
@@ -491,4 +762,4 @@ class Films(flow.Map):
 
 # TODO
 
-# file_category a corriger
\ No newline at end of file
+# file_category a corriger
