@@ -436,7 +436,10 @@ class Assets(flow.Map):
         return Asset
 
 class AssetFamily(flow.Object):
-    assets = flow.Child(Assets).ui(expanded=True)
+    assets = flow.Child(Assets).ui(
+        expanded=True,
+        show_filter=True,
+    )
 
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
@@ -451,7 +454,10 @@ class AssetFamilies(flow.Map):
 
 class AssetType(flow.Object):
 
-    asset_families = flow.Child(AssetFamilies).ui(expanded=True)
+    asset_families = flow.Child(AssetFamilies).ui(
+        expanded=True,
+        show_filter=True,
+    )
     utils = flow.Child(Utils).ui(expanded=False)
 
     def get_default_contextual_edits(self, context_name):
@@ -591,7 +597,10 @@ class CreateAssetAction(flow.Action):
 
 class AssetLib(flow.Object):
 
-    asset_types = flow.Child(AssetTypes).ui(expanded=True)
+    asset_types = flow.Child(AssetTypes).ui(
+        expanded=True,
+        show_filter=True,
+    )
     add_asset = flow.Child(CreateAssetAction).ui(
         label="Add asset"
     )
@@ -599,3 +608,7 @@ class AssetLib(flow.Object):
     def get_default_contextual_edits(self, context_name):
         if context_name == "settings":
             return dict(file_category="LIB")
+
+    def _fill_ui(self, ui):
+        if self.root().project().show_login_page():
+            ui["custom_page"] = "libreflow.baseflow.LoginPageWidget"
